@@ -3,7 +3,7 @@
 
 __author__ = """Jesus del Hoyo Munoz / Luis Miguel Sanchez Brea"""
 __email__ = 'jhoyo@ucm.es'
-__version__ = '1.0.4'
+__version__ = '1.0.5'
 
 import numpy as np
 import scipy as sp
@@ -29,3 +29,14 @@ eps = 1e-6
 num_decimals = 4
 
 number_types = (int, float, complex, np.int32, np.float64)
+
+shapes = {}
+shapes["Jones_vector"] = [2]
+shapes["Jones_matrix"] = [2, 2]
+shapes["Stokes"] = [4]
+shapes["Mueller"] = [4, 4]
+sizes = {}
+sizes["Jones_vector"] = 2
+sizes["Jones_matrix"] = 4
+sizes["Stokes"] = 4
+sizes["Mueller"] = 16
