@@ -101,8 +101,9 @@ def draw_ellipse(E,
     if E._type == 'Jones_vector':
         is_linear = E.checks.is_linear(shape=False, out_number=False)
     else:
-        is_linear = E.checks.is_linear(
-            shape=False, out_number=False, use_nan=False)
+        is_linear = E.checks.is_linear(shape=False,
+                                       out_number=False,
+                                       use_nan=False)
     # Create the electric field distributions
     Ex = E0X * np.cos(Angles + Phase)
     Ey = E0Y * np.cos(Angles + Phase + Delay)
@@ -220,30 +221,31 @@ def draw_ellipse(E,
                 string = str(list(np.unravel_index(ind, E.shape)))
         # Plot the probability distribution
         if depol_prob and is_depol[ind]:
-            IDimage = axis.imshow(
-                prob[ind, :, :],
-                interpolation='bilinear',
-                aspect='equal',
-                origin='lower',
-                extent=[-limit, limit, -limit, limit])
+            IDimage = axis.imshow(prob[ind, :, :],
+                                  interpolation='bilinear',
+                                  aspect='equal',
+                                  origin='lower',
+                                  extent=[-limit, limit, -limit, limit])
             # axis = axis[0]
         # Plot the curve
         if is_pol[ind]:
             axis.plot(Ex[ind, :], Ey[ind, :], lw=2, label=string, color=color)
             if draw_arrow and ~is_linear[ind]:
-                axis.arrow(
-                    Ex[ind, 0],
-                    Ey[ind, 0],
-                    Ex[ind, 4] - Ex[ind, 0],
-                    Ey[ind, 4] - Ey[ind, 0],
-                    width=0,
-                    head_width=0.075 * limit,
-                    linewidth=0,
-                    color=color,
-                    length_includes_head=True)
+                axis.arrow(Ex[ind, 0],
+                           Ey[ind, 0],
+                           Ex[ind, 4] - Ex[ind, 0],
+                           Ey[ind, 4] - Ey[ind, 0],
+                           width=0,
+                           head_width=0.075 * limit,
+                           linewidth=0,
+                           color=color,
+                           length_includes_head=True)
         elif E._type == 'Stokes' and depol_central and ~is_depol[ind]:
-            axis.plot(
-                np.zeros(2), np.zeros(2), lw=2, label=string, color=color)
+            axis.plot(np.zeros(2),
+                      np.zeros(2),
+                      lw=2,
+                      label=string,
+                      color=color)
         elif depol_central or depol_prob:
             print('Field {} is empty.'.format(string))
         else:
@@ -251,15 +253,15 @@ def draw_ellipse(E,
         # Add the depolarization for Stokes vectors
         if E._type == 'Stokes' and depol_central and is_depol[ind]:
             axis.plot(Exu[ind, :], Eyu[ind, :], lw=1.5, color=color, ls='--')
-        if E._type == 'Stokes' and depol_contour and is_pol[ind] and is_depol[ind]:
-            CS = axis.contour(
-                x,
-                x,
-                prob[ind, :, :],
-                contour_levels,
-                colors=(color),
-                linewidths=1.5,
-                linestyles=line_styles)
+        if E._type == 'Stokes' and depol_contour and is_pol[ind] and is_depol[
+                ind]:
+            CS = axis.contour(x,
+                              x,
+                              prob[ind, :, :],
+                              contour_levels,
+                              colors=(color),
+                              linewidths=1.5,
+                              linestyles=line_styles)
             # linestyles=('dashdot'))
         # Additions to figure
         if indC == Ncurves - 1:
@@ -407,7 +409,21 @@ def draw_ellipse(E,
 #     ax.set_zlim3d([zmean - 1 * plot_radius, zmean + 1 * plot_radius])
 
 
-def draw_poincare(S, fig=None, figsize=(6,6), draw_axes=True, draw_guides=True, kind="scatter", depol=False, param=None, subplots=False, in_degrees=False, log=False, hover="components", colormap="Blackbody", show_fig=False):
+def draw_poincare(S,
+                  fig=None,
+                  figsize=(6, 6),
+                  draw_axes=True,
+                  draw_guides=True,
+                  kind="scatter",
+                  depol=False,
+                  param=None,
+                  subplots=False,
+                  in_degrees=False,
+                  log=False,
+                  color_limits=(None, None),
+                  hover="components",
+                  colormap="Blackbody",
+                  show_fig=False):
     """Function to draw the Jones or Stokes vectors in the Poincare sphere using plotly.
 
     Args:
@@ -422,6 +438,7 @@ def draw_poincare(S, fig=None, figsize=(6,6), draw_axes=True, draw_guides=True,
         subplots (bool): If True, it tries to use the first two dimensions as rows and columns. If method == 'surf', this is mandatory. Default: False.
         in_degrees (bool): If True, transforms the parameters to degrees. Default: False.
         log (bool): If True, it calculates it in logarithmic scale. Default: False.
+        color_limits (float, float): limits in color
         hover (str): Choose between 'components' (S1, S2, S3) or 'angles' (azimuth, ellipticity). Default: 'components'.
         colormap (str): Colormap of the plots. Default: 'Blackbody'.
         show_fig (bool): If True, the figure is inmediately plotted. Default: False.
@@ -433,19 +450,25 @@ def draw_poincare(S, fig=None, figsize=(6,6), draw_axes=True, draw_guides=True,
     if hover == "angles":
         if kind == "surf":
             hover = "components"
-            print("WARNING: Plotly has an issue in the hover of surfaces. Hover mode changed to components.")
+            print(
+                "WARNING: Plotly has an issue in the hover of surfaces. Hover mode changed to components."
+            )
         else:
-            print("WARNING: Plotly has an issue in the hover of surfaces. Values are not correct.")
+            print(
+                "WARNING: Plotly has an issue in the hover of surfaces. Values are not correct."
+            )
     # Allow several figures. Indices: [row, column, trace, dots inside trace]
     S = S.copy()
     Ndim = S.ndim
     if kind == "surf":
         # Surf requires pseudo-1D, 2D, and higher dimensions to be separated in subplots
         if Ndim > 4 or Ndim < 1:
-            raise ValueError("Object has a wrong number of dimensions ({})".format(Ndim))
+            raise ValueError(
+                "Object has a wrong number of dimensions ({})".format(Ndim))
         elif Ndim == 1:
             if param is None:
-                raise ValueError("Object has a too few dimensions ({})".format(Ndim))
+                raise ValueError(
+                    "Object has a too few dimensions ({})".format(Ndim))
             else:
                 # TODO: Interpolate
                 Nrows = 1
@@ -496,71 +519,181 @@ def draw_poincare(S, fig=None, figsize=(6,6), draw_axes=True, draw_guides=True,
     if param is not None and not isinstance(param, str):
         param = np.reshape(param, S.shape)
 
-
     # Figure and figure options
     add_auxiliar = False
     if fig is None:
         specs = [[{'type': 'surface'}] * Ncolumns] * Nrows
         fig = make_subplots(rows=Nrows, cols=Ncolumns, specs=specs)
         add_auxiliar = True
-    lighting = dict(ambient=0.9, diffuse=0., roughness = 0.5, specular=0.05, fresnel=0.2)
+    lighting = dict(ambient=0.9,
+                    diffuse=0.,
+                    roughness=0.5,
+                    specular=0.05,
+                    fresnel=0.2)
     if hover == 'components':
-        hovertemplate = "S1: %{x:.3f}<br>S2: %{y:.3f}<br>S3: %{z:.3f}"
+        hovertemplate = "S1: %{x:.3f}<br>S2: %{y:.3f}<br>S3: %{z:.3f}<br>Parameter: %{customdata:.3f}"
     else:
         hovertemplate = "Az: %{customdata[0]:.1f}<br>El: %{customdata[1]:.1f}"
 
     # Axes
     if draw_axes and add_auxiliar:
+
         line_p = dict(width=6, color="red")
         line_m = dict(width=6, color="red", dash="dash")
         marker = dict(size=1)
-        axis_px = go.Scatter3d(x=[0,1.2], y=[0,0], z=[0,0], line=line_p, marker=marker, hoverinfo="skip", name="$S_1$")
-        axis_mx = go.Scatter3d(x=[0,-1.2], y=[0,0], z=[0,0], line=line_m, marker=marker, hoverinfo="skip", name="$-S_1$")
+        axis_px = go.Scatter3d(x=[0, 1.2],
+                               y=[0, 0],
+                               z=[0, 0],
+                               line=line_p,
+                               marker=marker,
+                               hoverinfo="skip",
+                               name="S1")
+        axis_mx = go.Scatter3d(x=[0, -1.2],
+                               y=[0, 0],
+                               z=[0, 0],
+                               line=line_m,
+                               marker=marker,
+                               hoverinfo="skip",
+                               name="-S1")
         line_p["color"], line_m["color"] = ("green", "green")
-        axis_py = go.Scatter3d(x=[0,0], y=[0,1.2], z=[0,0], line=line_p, marker=marker, hoverinfo="skip", name="$S_2$")
-        axis_my = go.Scatter3d(x=[0,0], y=[0,-1.2], z=[0,0], line=line_m, marker=marker, hoverinfo="skip", name="$-S_2$")
+        axis_py = go.Scatter3d(x=[0, 0],
+                               y=[0, 1.2],
+                               z=[0, 0],
+                               line=line_p,
+                               marker=marker,
+                               hoverinfo="skip",
+                               name="S2")
+        axis_my = go.Scatter3d(x=[0, 0],
+                               y=[0, -1.2],
+                               z=[0, 0],
+                               line=line_m,
+                               marker=marker,
+                               hoverinfo="skip",
+                               name="-S2")
         line_p["color"], line_m["color"] = ("blue", "blue")
-        axis_pz = go.Scatter3d(x=[0,0], y=[0,0], z=[0,1.2], line=line_p, marker=marker, hoverinfo="skip", name="$S_3$")
-        axis_mz = go.Scatter3d(x=[0,0], y=[0,0], z=[0,-1.2], line=line_m, marker=marker, hoverinfo="skip", name="$-S_3$")
-        size=15
+        axis_pz = go.Scatter3d(x=[0, 0],
+                               y=[0, 0],
+                               z=[0, 1.2],
+                               line=line_p,
+                               marker=marker,
+                               hoverinfo="skip",
+                               name="S3")
+        axis_mz = go.Scatter3d(x=[0, 0],
+                               y=[0, 0],
+                               z=[0, -1.2],
+                               line=line_m,
+                               marker=marker,
+                               hoverinfo="skip",
+                               name="-S3")
+        size = 30
         annotations = [
-            dict(showarrow=False, x=1.2, y=0, z=0, text="$S_1$", font=dict(color="red", size=size), xshift=15, yshift=15),
-            dict(showarrow=False, x=-1.2, y=0, z=0, text="$-S_1$", font=dict(color="red", size=size), xshift=-15, yshift=-15),
-            dict(showarrow=False, x=0, y=1.2, z=0, text="$S_2$", font=dict(color="green", size=size), xshift=15, yshift=15),
-            dict(showarrow=False, x=0, y=-1.2, z=0, text="$-S_2$", font=dict(color="green", size=size), xshift=-15, yshift=-15),
-            dict(showarrow=False, x=0, y=0, z=1.2, text="$S_3$", font=dict(color="blue", size=size), xshift=15, yshift=15),
-            dict(showarrow=False, x=0, y=0, z=-1.2, text="$-S_3$", font=dict(color="blue", size=size), xshift=-15, yshift=-15),
+            dict(showarrow=False,
+                 x=1.2,
+                 y=0,
+                 z=0,
+                 text="S1",
+                 font=dict(color="red", size=size),
+                 xshift=15,
+                 yshift=15),
+            dict(showarrow=False,
+                 x=-1.2,
+                 y=0,
+                 z=0,
+                 text="-S1",
+                 font=dict(color="red", size=size),
+                 xshift=-15,
+                 yshift=-15),
+            dict(showarrow=False,
+                 x=0,
+                 y=1.2,
+                 z=0,
+                 text="S2",
+                 font=dict(color="green", size=size),
+                 xshift=15,
+                 yshift=15),
+            dict(showarrow=False,
+                 x=0,
+                 y=-1.2,
+                 z=0,
+                 text="-S2",
+                 font=dict(color="green", size=size),
+                 xshift=-15,
+                 yshift=-15),
+            dict(showarrow=False,
+                 x=0,
+                 y=0,
+                 z=1.2,
+                 text="S3",
+                 font=dict(color="blue", size=size),
+                 xshift=15,
+                 yshift=15),
+            dict(showarrow=False,
+                 x=0,
+                 y=0,
+                 z=-1.2,
+                 text="-S3",
+                 font=dict(color="blue", size=size),
+                 xshift=-15,
+                 yshift=-15),
         ]
     else:
         annotations = []
 
     # Curves
     if draw_guides and add_auxiliar:
-        angle = np.linspace(0, 360*degrees, 361)
+        angle = np.linspace(0, 360 * degrees, 361)
         line = dict(width=2, color="darkslategrey", dash="dashdot")
         x = np.sin(angle)
         y = np.cos(angle)
         z = np.zeros_like(angle)
-        circle_z = go.Scatter3d(x=x, y=y, z=z, line=line, mode="lines", name="S3=0")
+        circle_z = go.Scatter3d(x=x,
+                                y=y,
+                                z=z,
+                                line=line,
+                                mode="lines",
+                                name="S3=0")
         x = np.sin(angle)
         z = np.cos(angle)
         y = np.zeros_like(angle)
-        circle_y = go.Scatter3d(x=x, y=y, z=z, line=line, mode="lines", name="S2=0")
+        circle_y = go.Scatter3d(x=x,
+                                y=y,
+                                z=z,
+                                line=line,
+                                mode="lines",
+                                name="S2=0")
         y = np.sin(angle)
         z = np.cos(angle)
         x = np.zeros_like(angle)
-        circle_x = go.Scatter3d(x=x, y=y, z=z, line=line, mode="lines", name="S1=0")
+        circle_x = go.Scatter3d(x=x,
+                                y=y,
+                                z=z,
+                                line=line,
+                                mode="lines",
+                                name="S1=0")
 
     # Poincare sphere
     if kind != "surf" and add_auxiliar:
         # az, el = np.mgrid[0:180*degrees:100j, -45*degrees:45*degrees:100j]
-        el, az = np.mgrid[-45*degrees:45*degrees:100j, 0:180*degrees:100j]
+        el, az = np.mgrid[-45 * degrees:45 * degrees:100j,
+                          0:180 * degrees:100j]
         x, y, z = azel_2_xyz(az, el)
-        customdata = np.dstack((az/degrees, el/degrees)) if hover == "angles" else []
+        customdata = np.dstack(
+            (az / degrees, el / degrees)) if hover == "angles" else []
         level = 0.2
-        Psphere = go.Surface(x=x, y=y, z=z, surfacecolor=np.ones_like(x)*level, cmin=0, cmax=1, opacity=0.5,
-            colorscale="Greys", showscale=False, lighting=lighting, customdata=customdata,
-            hovertemplate=hovertemplate, name="Sphere", showlegend=False)
+        Psphere = go.Surface(x=x,
+                             y=y,
+                             z=z,
+                             surfacecolor=np.ones_like(x) * level,
+                             cmin=0,
+                             cmax=1,
+                             opacity=1,
+                             colorscale="Greys",
+                             showscale=False,
+                             lighting=lighting,
+                             customdata=customdata,
+                             hovertemplate=hovertemplate,
+                             name="Sphere",
+                             showlegend=False)
 
     # Loop in rows
     for indR, Srow in enumerate(S):
@@ -568,21 +701,27 @@ def draw_poincare(S, fig=None, figsize=(6,6), draw_axes=True, draw_guides=True,
 
             # Axes
             if draw_axes and add_auxiliar:
-                fig.add_traces([axis_px, axis_mx, axis_py, axis_my, axis_pz, axis_mz], rows=indR+1, cols=indC+1)
+                fig.add_traces(
+                    [axis_px, axis_mx, axis_py, axis_my, axis_pz, axis_mz],
+                    rows=indR + 1,
+                    cols=indC + 1)
 
             # Curves
             if draw_guides and add_auxiliar:
-                fig.add_traces([circle_z, circle_y, circle_x], rows=indR+1, cols=indC+1)
+                fig.add_traces([circle_z, circle_y, circle_x],
+                               rows=indR + 1,
+                               cols=indC + 1)
 
             # Loop in traces
             for indT, Strace in enumerate(Scol):
 
                 # Parameter
                 if isinstance(param, str):
-                    colorbar = dict(title=param)
-                    Scolor = eval("Strace.parameters." + param + "(out_number=False)")
+                    colorbar = dict(title=param,orientation='h',ticklabelposition='outside bottom',y=-0.1)
+                    Scolor = eval("Strace.parameters." + param +
+                                  "(out_number=False)")
                 elif param is not None:
-                    colorbar = dict(title=S.name)
+                    colorbar = dict(title=S.name,orientation='h',ticklabelposition='outside bottom',y=-0.1,tickfont=dict(size=30))
                     Scolor = param[indR, indC, indT, ...]
                 else:
                     colorbar = {}
@@ -595,6 +734,7 @@ def draw_poincare(S, fig=None, figsize=(6,6), draw_axes=True, draw_guides=True,
                             Scolor[cond] = np.min(Scolor) / 10
                         Scolor = np.log10(Scolor)
                         colorbar["title"] = "log(" + colorbar["title"] + ")"
+                        
                     elif in_degrees:
                         Scolor = Scolor / degrees
                         colorbar["title"] += " (deg)"
@@ -603,48 +743,121 @@ def draw_poincare(S, fig=None, figsize=(6,6), draw_axes=True, draw_guides=True,
                 if kind != "surf":
                     # Poincare sphere
                     if add_auxiliar:
-                        fig.add_trace(Psphere, row=indR+1, col=indC+1)
+                        fig.add_trace(Psphere, row=indR + 1, col=indC + 1)
 
                     # Data
                     if kind in ("scatter", "scatterline"):
-                        x, y, z, az, el, _ = obj_2_xyz(Strace, in_degrees=True, depol=depol)
-                        customdata = np.squeeze(np.dstack((az, el))) if hover == "angles" else []
+                        x, y, z, az, el, _ = obj_2_xyz(Strace,
+                                                       in_degrees=True,
+                                                       depol=depol)
+                        customdata = np.squeeze(np.dstack(
+                            (az, el))) if hover == "angles" else []
                         if kind == "scatter":
-                            marker = dict(size=10, color=Scolor, colorbar=colorbar, colorscale=colormap)
+                            marker = dict(size=10,
+                                          color=Scolor,
+                                          colorbar=colorbar,
+                                          colorscale=colormap)
                         else:
-                            marker = dict(size=10, color=Scolor, colorscale=colormap)
-                        Fdata = go.Scatter3d(x=x, y=y, z=z, marker=marker, name=S.name,
-                                        mode="markers", customdata=customdata, hovertemplate=hovertemplate)
+                            marker = dict(size=10,
+                                          color=Scolor,
+                                          colorscale=colormap)
+                        Fdata = go.Scatter3d(x=x,
+                                             y=y,
+                                             z=z,
+                                             marker=marker,
+                                             name=S.name,
+                                             mode="markers",
+                                             customdata=customdata,
+                                             hovertemplate=hovertemplate)
                         # TODO: Should be possible to link colorbars. Not right now due to a bug in Plotly.
                         # marker = dict(size=10, color=Scolor, colorscale=colormap)
                         # Fdata = go.Scatter3d(x=x, y=y, z=z, marker=marker, name=S.name,
                         #                 mode="markers", customdata=customdata, hovertemplate=hovertemplate, coloraxis="coloraxis")
-                        fig.add_trace(Fdata, row=indR+1, col=indC+1)
+                        fig.add_trace(Fdata, row=indR + 1, col=indC + 1)
                     if kind in ("line", "scatterline"):
-                        x, y, z, az, el, Scolor = obj_2_xyz(Strace, DAinterp=1*degrees, in_degrees=True, param=Scolor, depol=depol)
-                        customdata = np.squeeze(np.dstack((az, el))) if hover == "angles" else []
-                        line = dict(width=8, color=Scolor, colorbar=colorbar, colorscale=colormap)
-                        Fdata = go.Scatter3d(x=x, y=y, z=z, line=line, name=S.name+" int.",
-                                        mode="lines", customdata=customdata, hovertemplate=hovertemplate)
-                        fig.add_trace(Fdata, row=indR+1, col=indC+1)
+                        x, y, z, az, el, Scolor = obj_2_xyz(Strace,
+                                                            DAinterp=1 *
+                                                            degrees,
+                                                            in_degrees=True,
+                                                            param=Scolor,
+                                                            depol=depol)
+                        customdata = np.squeeze(np.dstack(
+                            (az, el))) if hover == "angles" else []
+                        line = dict(width=8,
+                                    color=Scolor,
+                                    colorbar=colorbar,
+                                    colorscale=colormap)
+                        Fdata = go.Scatter3d(x=x,
+                                             y=y,
+                                             z=z,
+                                             line=line,
+                                             name=S.name + " int.",
+                                             mode="lines",
+                                             customdata=customdata,
+                                             hovertemplate=hovertemplate)
+                        fig.add_trace(Fdata, row=indR + 1, col=indC + 1)
 
                 else:
-                    x, y, z, az, el, Scolor = obj_2_xyz(Strace, in_degrees=True, param=Scolor, interp_to_surf=True)
+                    x, y, z, az, el, Scolor = obj_2_xyz(Strace,
+                                                        in_degrees=True,
+                                                        param=Scolor,
+                                                        interp_to_surf=True)
+                    
                     # customdata = [az[0,:], el[:, 0]] if hover == "angles" else None
-                    customdata = np.squeeze(np.stack((az, el), axis=-1)) if hover == "angles" else None
-                    # print(customdata[0], customdata[1])
-                    Psphere = go.Surface(x=x, y=y, z=z, surfacecolor=Scolor, opacity=1,
-                        colorscale=colormap, lighting=lighting, customdata=customdata,
-                        hovertemplate=hovertemplate, name=S.name, colorbar=colorbar)
-                    fig.add_trace(Psphere, row=indR+1, col=indC+1)
-
+                    customdata = np.squeeze(np.stack(
+                        (az, el), axis=-1)) if hover == "angles" else None
+                    customdata = np.stack((Scolor), axis=-1)
+                    if color_limits == None:
+                        Psphere = go.Surface(x=x,
+                                             y=y,
+                                             z=z,
+                                             surfacecolor=Scolor,
+                                             opacity=1,
+                                             colorscale=colormap,
+                                             lighting=lighting,
+                                             customdata=customdata,
+                                             hovertemplate=hovertemplate,
+                                             name=S.name,
+                                             colorbar=colorbar)
+                    else:
+                        cmin, cmax = color_limits
+                        if cmin is None:
+                            cmin = Scolor.min()
+                        if cmax is None:
+                            cmax = Scolor.max()
+                        Psphere = go.Surface(x=x,
+                                             y=y,
+                                             z=z,
+                                             surfacecolor=Scolor,
+                                             opacity=1,
+                                             colorscale=colormap,
+                                             showscale=True,
+                                             lighting=lighting,
+                                             customdata=customdata,
+                                             hovertemplate=hovertemplate,
+                                             name=S.name,
+                                             cmin=cmin,
+                                             cmax=cmax,
+                                             colorbar=colorbar)
+                    fig.add_trace(Psphere, row=indR + 1, col=indC + 1)
 
     # Plot figure
-    axis = dict(showbackground=False, showgrid=False, zeroline=False, visible=False)
-    camera = dict(up=dict(x=0, y=0, z=1), center=dict(x=0, y=0, z=0), eye=dict(x=0.75, y=0.75, z=0.75))
-    fig.update_layout(margin=dict(l=20, r=20, t=20, b=20), width=int(figsize[0]*100),
-                    height=int(figsize[1]*100), showlegend=False)
-    fig.update_scenes(annotations=annotations, xaxis=axis, yaxis=axis, zaxis=axis, camera=camera)
+    axis = dict(showbackground=False,
+                showgrid=False,
+                zeroline=False,
+                visible=False)
+    camera = dict(up=dict(x=0, y=0, z=1),
+                  center=dict(x=0, y=0, z=0),
+                  eye=dict(x=0.75, y=0.75, z=0.75))
+    fig.update_layout(margin=dict(l=20, r=20, t=20, b=20),
+                      width=int(figsize[0] * 100),
+                      height=int(figsize[1] * 100),
+                      showlegend=False)
+    fig.update_scenes(annotations=annotations,
+                      xaxis=axis,
+                      yaxis=axis,
+                      zaxis=axis,
+                      camera=camera)
     # TODO: Annotations do not render. Why???
 
     if show_fig:
