@@ -234,22 +234,47 @@ class Py_pol(object):
         """Create a Py_pol object from a list of numpy arrays of the correct size.
 
         Parameters:
-            l (list): list of np.ndarrays.
+            l (list): list of np.ndarrays, lists, tuples or Py_pol objects.
             shape_like (numpy.ndarray or py_pol object): Use the shape of this object. Default: None.
             shape (tuple or list): If no shape_like array is given, use this shape instead. Default: None.
 
         Returns:
             (Py_pol): Created object.
         """
-        # Preallocate memory
-        N = len(l)
-        M = np.squeeze(np.zeros(shapes[self.type] + [N]))
-        # Fill it
+        Saux = self.copy()
         for ind, elem in enumerate(l):
-            if self.type in ("Jones_vector", "Stokes"):
-                M[:, ind] = np.squeeze(elem)
+            # Get the data in an ordered way from any shape
+            if isinstance(elem, (np.ndarray, tuple, list)):
+                Saux.from_matrix(elem)
+            elif self.type == elem.type:
+                Saux = elem
             else:
-                M[:, :, ind] = np.squeeze(elem)
+                raise ValueError("New element of type {} can't be added to an object of type {}.".format(elem.type, self.type))
+            # Concatenate
+            if ind == 0:
+                M = Saux.M
+            else:
+                M = np.hstack((M, Saux.M))
+
+
+        # # Preallocate memory
+        # if isinstance(l[0], (np.ndarray, tuple, list)):
+        #     M = np.array(l[0])
+        #     if M.ndim == 1:
+        #         M = M.reshape((M.size, 1))
+        # else:
+        #     M = l[0].M
+        # # Fill it
+        # for elem in l[1:]:
+        #     if isinstance(elem, (np.ndarray, tuple, list)):
+        #         Maux = np.array(elem)
+        #         if Maux.ndim == 1:
+        #             Maux = Maux.reshape((Maux.size, 1))
+        #         M = np.hstack((M, Maux))
+        #     elif self.type == elem.type:
+        #         M = np.hstack((M, elem.M))
+        #     else:
+        #         raise ValueError("New element of type {} can't be added to an object of type {}.".format(elem.type, self.type))
         # Update
         self.from_matrix(M, shape=shape, shape_like=shape_like)
         return self
